# -*- coding: utf-8 -*-

import logging
import unittest
from types import FunctionType

from noseapp.core import loader
from noseapp.core import extensions
from noseapp.suite.context import SuiteContext
from noseapp.case.screenplay import ScreenPlayCase
from noseapp.case.base import make_test_case_class_from_function


logger = logging.getLogger(__name__)


def suite_is_mount(suite):
    """
    If suite is mounting to app then True else False.

    :param suite: suite instance
    :type suite: Suite

    :rtype: bool
    """
    return bool(suite.of_app)


class Suite(object):
    """
    Base Suite class for group or one TestCase.

    Usage::

        suite = Suite(__name__)

        @suite.register
        class Case(suite.TestCase):

            def runTest(self):
                pass

        @suite.register
        def my_test_case(case):
            case.assertTrue(True)

        @suite.register(simple=True)
        def simple_test():
            assert True
    """

    # Constant is pre set for require param
    DEFAULT_REQUIRE = None

    # This class is base class for making test case from function
    test_case_class = ScreenPlayCase

    def __init__(self, name, require=None, context=None):
        """
        :param name: suite name
        :type name: str

        :param require: extension names
        :type require: list

        :param context: context instance
        :type context: noseapp.suite.context.SuiteContext
        """
        # Suite name. Must be str only
        self.__name = name
        # of app name
        self.__of_app = None
        # Was suite built? True or False
        self.__is_build = False

        # Context of this instance
        self.__context = context or SuiteContext(
            list(self.DEFAULT_REQUIRE or []) + list(require or []),
        )

        # Push callbacks to context. They'll be called with run nose suite.
        self.__context.add_setup(lambda: self.setUp())
        self.__context.add_teardown(lambda: self.tearDown())

    @property
    def name(self):
        """
        Suite name.
        If suite is mounting to app then name
        is <app_name>.<suite_name> else <suite_name>.
        """
        if self.__of_app:
            return '{}.{}'.format(
                self.__of_app, self.__name,
            )

        return self.__name

    @property
    def of_app(self):
        """
        Name of mounted application.

        :rtype: str or None
        """
        return self.__of_app

    @property
    def context(self):
        """
        Suite context.

        :rtype: noseapp.suite.context.SuiteContext
        """
        return self.__context

    @property
    def is_build(self):
        """
        If suite is build return True else False.

        :rtype: bool
        """
        return self.__is_build

    @property
    def test_cases(self):
        """
        Test cases storage of context.

        :rtype: list
        """
        return self.__context.test_cases

    @property
    def require(self):
        """
        Extensions names storage of context.

        :rtype: list
        """
        return self.__context.require

    @property
    def status(self):
        """
        Get suite status.
        If suite is mounted to application
        then 'mounted' else 'unmounted'.

        :rtype: str
        """
        return 'mounted' if self.__of_app else 'unmounted'

    def mount_to_app(self, app):
        """
        :param app: application instance
        :type app: noseapp.app.base.NoseApp

        :raises: RuntimeError
        """
        if suite_is_mount(self):
            raise RuntimeError(
                '{} already mount to <NoseApp {}>. '.format(self, self.__of_app),
            )

        self.__of_app = app.name

        app.context.add_suite(self)

    @property
    def TestCase(self):
        """
        Get test case class from suite instance.
        Will be return a test_case_class alias.
        The alias is base class for making test case from function.

        :rtype: noseapp.case.screenplay.ScreenPlayCase
        """
        return self.test_case_class

    def setUp(self):
        """
        Callback. Will be called run before run suite.
        """
        pass

    def tearDown(self):
        """
        Callback. Will be called run after run suite.
        """
        pass

    @property
    def skip(self):
        """
        Proxy to unittest.skip.

        Example::

            class Test(suite.TestCase):

                @suite.skip('TODO: ...')
                def test(self):
                    pass

        :rtype: unittest.skip
        """
        return unittest.skip

    @property
    def skip_if(self):
        """
        Proxy to unittest.skipIf.

        :rtype: unittest.skipIf
        """
        return unittest.skipIf

    @property
    def skip_unless(self):
        """
        Proxy to unittest.skipUnless.

        :rtype: unittest.skipUnless
        """
        return unittest.skipUnless

    # unit test style
    skipIf = skip_if
    skipUnless = skip_unless

    def add_pre_run(self, func):
        """
        Add function for pre run test case to context.

        Example::

            @suite.add_pre_run
            def pre_run(case):
                case.do_something()

            # or

            suite.add_pre_run(lambda case: case.do_something())
        """
        self.__context.add_pre_run(func)
        return func

    def add_post_run(self, func):
        """
        Add function for post run test case to context.

        Example::

            @suite.add_post_run
            def post_run(case):
                case.do_something()

            # or

            suite.add_post_run(lambda case: case.do_something())
        """
        self.__context.add_post_run(func)
        return func

    def add_setup(self, func):
        """
        Add setup callback to context.

        Example::

            @suite.add_setup
            def setup():
                pass

            # or

            suite.add_setup(lambda: None)
        """
        self.__context.add_setup(func)
        return func

    def add_teardown(self, func):
        """
        Add teardown callback to context.

        Example::

            @suite.add_teardown
            def teardown():
                pass

            # or

            suite.add_teardown(lambda: None)
        """
        self.__context.add_teardown(func)
        return func

    def ext(self, name):
        """
        Get extension by name.
        Extensions will be available after build suite.

        Example::

            class MySuite(Suite):

                DEFAULT_REQUIRE = [
                    'extension',
                ]

                def setUp(self):
                    ext = self.ext('extension')

        :param name: extension name
        :type name: str

        :raises: RuntimeError, noseapp.core.extensions.ExtensionNotRequired
        """
        if not self.__is_build:
            raise RuntimeError(
                'Extensions is not available. Suite is not building.',
            )

        if name not in self.__context.require:
            raise extensions.ExtensionNotRequired(name)

        return self.__context.extensions.get(name)

    def register(self, cls=None, **kwargs):
        """
        Add test case to context.

        Example::

            @suite.register
            class TestCase(suite.TestCase):

                def test(self):
                    pass

            # Skip test
            @suite.register(skip='TODO: ...')
            def test(case):
                pass

            # Require extension for test case only
            @suite.register(require=['extension'])
            def test_require_ex(case):
                ext = case.ext('extension')

        :param cls: test case class
        :type cls: noseapp.case.TestCase or function

        :param simple: use for simple test function
        :type simple: bool

        :param skip: if to set then skip case
        :type skip: str

        :param require: extensions names
        :type require: list

        :raises: TypeError, RuntimeError
        :rtype: cls
        """
        if not cls and not kwargs:
            raise TypeError('cls param or **kwargs is required')
        elif cls and kwargs:
            raise TypeError('**kwargs can not be used with cls param')

        def wrapped(_class, simple=False, skip=None, require=None):
            if type(_class) == FunctionType:
                _class = make_test_case_class_from_function(
                    _class,
                    simple=simple,
                    base_class=self.TestCase,
                )

            if skip:
                unittest.skip(skip)(_class)

            if hasattr(_class, 'REQUIRE'):
                raise RuntimeError('"REQUIRE" attribute can not be pre set')

            if require:
                setattr(_class, 'REQUIRE', require)

            self.__context.add_test_case(_class)

            logger.debug(
                'Register test case "{}" on {}'.format(
                    _class.__name__, repr(self),
                ),
            )

            return _class

        if cls:
            return wrapped(cls)

        def wrapper(_class):
            return wrapped(_class, **kwargs)

        return wrapper

    def get_map(self):
        """
        Get map of test classes.

        ::

            {
                'class name': {
                    'cls': link_to_class,
                    'tests': {
                        'method name': link_to_test_method,
                    },
                },
            }

        :rtype: dict
        """
        mp = {}

        for case in self.__context.test_cases:
            mp[case.__name__] = {
                'cls': case,
                'tests': dict(
                    (atr, getattr(case, atr))
                    for atr in dir(case)
                    if atr.startswith(loader.TEST_NAME_PREFIX)
                    or
                    atr == loader.DEFAULT_TEST_NAME,
                ),
            }

        return mp

    def __call__(self,
                 program_data,
                 shuffle=None,
                 case_name=None,
                 method_name=None):
        """
        Build suite. After call suite instance will be
        created instance of nose.suite.ContextSuite

        :param program_data: instance of ProgramData
        :type program_data: noseapp.core.program.ProgramData

        :param shuffle: callable object for randomize test case list
        :param case_name: test name for build
        :param method_name: test case method name for build

        :raises: RuntimeError
        :rtype: noseapp.core.suite.base.BaseSuite
        """
        if not self.__of_app:
            raise RuntimeError('Suite can not be building, not mounted to app')

        self.__is_build = True

        for ext_name in self.context.require:
            ext = extensions.get(ext_name)
            self.context.add_extension(ext_name, ext)

        if callable(shuffle):
            shuffle(self.__context.test_cases)

        def make_suites():
            suites = []

            if case_name:
                case = loader.load_case_from_suite(case_name, self)
                tests = loader.load_tests_from_test_case(
                    case.mount_to_suite(self),
                    method_name=method_name,
                )

                suites.append(
                    program_data.suite_class(
                        tests,
                        context=case,
                        config=program_data.config,
                        pre_run_handlers=self.__context.pre_run,
                        post_run_handlers=self.__context.post_run,
                        resultProxy=program_data.result_proxy_factory,
                    ),
                )
            else:
                for case in self.__context.test_cases:
                    tests = loader.load_tests_from_test_case(
                        case.mount_to_suite(self),
                    )

                    suites.append(
                        program_data.suite_class(
                            tests,
                            context=case,
                            config=program_data.config,
                            pre_run_handlers=self.__context.pre_run,
                            post_run_handlers=self.__context.post_run,
                            resultProxy=program_data.result_proxy_factory,
                        ),
                    )

            return suites

        return program_data.suite_class(
            make_suites(),
            context=self.__context,
            config=program_data.config,
        )

    def __repr__(self):
        return '<Suite({}): {}>'.format(self.status, self.name)



# -*- coding: utf8 -*-

import os
import sys
from argparse import ArgumentError

from nose.proxy import ResultProxyFactory
from nose.core import TestProgram as BaseTestProgram

from noseapp.core import collector
from noseapp.core import extensions
from noseapp.app.context import app_callback


def prepare_argv(argv, plugins):
    """
    :type argv: list
    """
    argv = sys.argv + (argv or [])

    for arg in argv:
        if '--processes' in arg:
            raise ArgumentError(None, 'Option "--processes" is deprecated')
        if arg == '--ls':
            os.environ.setdefault('NOSE_NOCAPTURE', '1')

    argv.extend(('--with-{}'.format(p.name) for p in plugins))

    return argv


class ProgramData(object):
    """
    Class is merging data of test program and application
    """

    def __init__(self, program, app, argv=None):
        from noseapp.app import config

        # Push options from parser to application
        app.options = app.config_class(
            config.load(program.config.options),
        )
        # Init application is here
        app_callback(app, 'setUpApp')

        self.__argv = argv or sys.argv
        self.__class_factory = app.class_factory(program.config.options)

        self.__app = app
        self.__program = program

        self.__result_proxy_factory = ResultProxyFactory(
            config=self.__program.config,
        )

    @property
    def argv(self):
        return self.__argv

    @property
    def suites(self):
        return self.__app.suites

    @property
    def config(self):
        return self.__program.config

    @property
    def context(self):
        return self.__app.context

    @property
    def suite_class(self):
        return self.__class_factory.suite_class

    @property
    def runner_class(self):
        return self.__class_factory.runner_class

    @property
    def result_proxy_factory(self):
        return self.__result_proxy_factory

    def build_suite(self):
        """
        Collect and build suites
        """
        with extensions.installation():
            suites = collector.collect(
                self,
                collector_class=self.__app.collector_class,
            )

        return suites


class TestProgram(BaseTestProgram):

    def __init__(self, app, argv=None, exit=True):
        argv = prepare_argv(argv, app.plugins)

        super(TestProgram, self).__init__(
            argv=argv,
            exit=exit,
            addplugins=app.plugins,
        )

        self.testLoader = None  # We are not using that
        self.data = ProgramData(self, app, argv=argv)

    # disarm base class
    def createTests(self):
        pass

    def runTests(self):
        pass

    def run(self):
        """
        Perform test program
        """
        self.testRunner = self.data.runner_class(
            stream=self.config.stream,
            verbosity=self.config.verbosity,
            config=self.config,
        )

        plug_runner = self.config.plugins.prepareTestRunner(self.testRunner)

        if plug_runner is not None:
            self.testRunner = plug_runner

        suites = self.data.build_suite()

        if not suites:
            raise RuntimeError('No suites for running')

        result = self.testRunner.run(suites)
        self.success = result.wasSuccessful()

        if self.exit:
            sys.exit(not self.success)

        return self.success



# -*- coding: utf-8 -*-

"""
Application is entry point to your test program.
You can configure and expand your program from application.
"""

import os
import sys
import logging
from contextlib import contextmanager

from noseapp.core import loader
from noseapp.core import extensions
from noseapp.core import TestProgram
from noseapp.app.context import AppContext
from noseapp.app.context import app_callback
from noseapp.app.context import merge_context
from noseapp.core.factory import ClassFactory
from noseapp.core.collector import CollectSuite
from noseapp.app.config import Config as AppConfig
from noseapp.core.constants import CONFIG_CHECKOUT_ENV
from noseapp.plugins.configure import AppConfigurePlugin


logger = logging.getLogger(__name__)


DEFAULT_PLUGINS = [
    AppConfigurePlugin(),
]

APP_CONFIG = os.getenv(CONFIG_CHECKOUT_ENV, None)


class NoseApp(object):
    """
    Base application class

    Usage:

        >>> app = NoseApp('my_app')
        >>> app.load_suites('/path/to/suites')
        >>> app.run()

    """

    config_class = AppConfig
    program_class = TestProgram
    class_factory = ClassFactory
    collector_class = CollectSuite

    def __init__(self,
                 name,
                 argv=None,
                 exit=True,
                 config=None,
                 plugins=None,
                 context=None,
                 sub_apps=None,
                 suites_path=None,
                 is_sub_app=False):
        """
        :param name: application name.
        application name is prefix to suite name.
         namespace pattern: <application_name>.<suite_name>:<test_case_name>

        for example:

            >>> from noseapp import Suite

            >>> app = NoseApp('my_app')
            >>> suite = Suite('my_suite')
            >>> suite.name
            >>> 'my_suite'

            >>> app.register_suite(suite)
            >>> suite.name
            >>> 'my_app.my_suite'

        :type name: str

        :param argv: argv to sys.argv
        :type argv: list

        :param exit: do exit after run?
        :type exit: bool

        :param config: import path or absolute file path
        for example:
            * config='project.etc.config'
            * config='/home/user/project/etc/config.py'
        :type config: str

        :param plugins: plugins instances
        :type plugins: list or tuple

        :param context: inctance of AppContext.
         you can create own context instance to run.
        :type context: noseapp.app.context.AppContext

        :param suites_path: path to directory where contains suites
        :type suites_path: str

        :param is_sub_app: is application a sub application?
        :type is_sub_app: bool

        :param sub_apps: sub application instances. for master application only.
        :type sub_apps: list or tuple
        """
        if sub_apps and is_sub_app:
            raise RuntimeError(
                '"{}" can not be maser app. The application is sub app.'.format(self),
            )

        # Initialization config storage
        self.config = self.config_class.from_path(APP_CONFIG or config)

        # Application name. Must be str only.
        self.__name = name

        # Command line options. Will be set after test program initialization.
        self.__options = None

        # If app is sub app that's True else False
        self.__is_sub_app = is_sub_app

        # Sub application list
        self.__sub_apps = list(sub_apps or [])

        # Context will be inject to nose suite
        self.__context = context or AppContext()

        # Add callbacks to context
        self.__context.add_setup(lambda: self.setUp())
        self.__context.add_teardown(lambda: self.tearDown())

        # Saved support nose plugins and add supported self plugins.
        self.__context.plugins.extend(list(plugins or []))

        if self.is_master_app:
            # Default plugins for master app only
            self.__context.plugins.extend(DEFAULT_PLUGINS)
            # Merge context of sub applications
            merge_context(self,
                          merge_setup=True,
                          merge_suites=True,
                          merge_plugins=True,
                          merge_teardown=True)
            # Program object initialization. Will be run later.
            self.__test_program = self.program_class(self, argv=argv, exit=exit)

            # If we are having path to suites dir then make load suites
            if suites_path is not None:
                self.load_suites(suites_path)

    @classmethod
    def as_master_app(cls, name, *sub_apps, **kwargs):
        """
        Alternative constructor.
        Create application as master application.

        Example::

            sub_app = NoseApp.as_sub_app('my_sub_app')
            app = NoseApp.as_master_app('my_app', sub_app, config='etc.config')

            sub_app.load_suites('/path/to/suites')
            app.load_suites('/path/to/suites', merge_suites=True)
        """
        kwargs.update(is_sub_app=False)
        kwargs.setdefault('sub_apps', sub_apps)

        return cls(name, **kwargs)

    @classmethod
    def as_sub_app(cls, name, **kwargs):
        """
        Alternative constructor.
        Create application as sub application. See as_master_app.
        """
        kwargs.update(
            sub_apps=None,
            is_sub_app=True,
        )

        return cls(name, **kwargs)

    def setUpApp(self):
        """
        For setup extensions end prepare program.
        Will be called after creating instance.
        """
        pass

    def tearDownApp(self):
        """
        Callback for destruct application.
        Will be called after exit from test program.
        """
        pass

    def setUp(self):
        """
        Do it something before run suites.
        Callback will be called before run application.
        """
        pass

    def tearDown(self):
        """
        Do it something after run suites.
        Callback will be called after run application
        """
        pass

    def addOptions(self, parser):
        """
        Add options to command line interface.

        Example::

            from optparse import OptionGroup


            class MyApp(NoseApp):

                def addOptions(parser):
                    group = OptionGroup(parser, 'My Application options')

                    group.add_option(
                        '--project-url',
                        dest='project_url',
                        default=None,
                        help='Web URL',
                    )

                    parser.add_option_group(group)

        :param parser: options parser
        :type parser: optparse.OptionParser
        """
        pass

    @property
    def name(self):
        """
        Application name
        """
        return self.__name

    @property
    def options(self):
        """
        Command line options. Will be available after create instance.

        :raises: RuntimeError
        """
        if self.__options is None:
            raise RuntimeError(
                'Options not found. Working outside app context.',
            )

        return self.__options

    @options.setter
    def options(self, options):
        assert isinstance(options, self.config_class),\
            'options is not instance of "{}"'.format(self.config_class.__name__)

        if self.is_master_app:
            for sub_app in self.__sub_apps:
                sub_app.options = options

        self.__options = options

    @property
    def context(self):
        """
        Context of application instance

        :rtype: noseapp.app.context.AppContext
        """
        return self.__context

    @property
    def plugins(self):
        """
        Plugins storage of context

        :rtype: list
        """
        return self.__context.plugins

    @property
    def is_sub_app(self):
        """
        Flag to definition: is it sub application?

        :rtype: bool
        """
        return self.__is_sub_app

    @property
    def is_master_app(self):
        """
        Flag to definition: is it master application?

        :rtype: bool
        """
        return not self.__is_sub_app

    @property
    def suites(self):
        """
        Suites storage of context

        :rtype: list
        """
        return self.__context.suites

    @property
    def sub_apps(self):
        """
        Sub application storage

        :rtype: list
        """
        return self.__sub_apps

    @property
    def status(self):
        """
        Status of application.

        Example::

            >>> app = NoseApp.as_master_app('my_app')
            >>> app.status
            >>> 'master'

            >>> app = NoseApp.as_sub_app('my_app')
            >>> app.status
            >>> 'sub'
        """
        return 'sub' if self.__is_sub_app else 'master'

    def add_setup(self, func):
        """
        Add setup callback to context.

        Example::

            app = NoseApp('my_app')

            @app.add_setup
            def setup():
                # do something
                pass

            # or

            app.add_setup(lambda: None)

        """
        self.__context.add_setup(func)
        return func

    def add_teardown(self, func):
        """
        Add teardown callback to context.

        Example::

            app = NoseApp('my_app')

            @app.add_teardown
            def teardown():
                # do something
                pass

            # or

            app.add_teardown(lambda: None)
        """
        self.__context.add_teardown(func)
        return func

    @staticmethod
    def shared_extension(name=None, cls=None, args=None, kwargs=None):
        """
        Shared extension to Suite and TestCase instances.
        Use require param on noseapp.Suite class for connect.

        Example::

            import random

            app = NoseApp('my_app')
            app.shared_extension(name='random_int', cls=random.randint, args=(0, 100))

        :param name: extension name
        :type name: str

        :param cls: extension class or callable object
        :type cls: object, callable

        :param args: cls init args
        :type args: tuple

        :param kwargs: cls init kwargs
        :type kwargs: dict

        :raises: ValueError, AttributeError
        """
        if cls is None:
            raise ValueError('cls param is required')

        if name is None and not hasattr(cls, 'name'):
            raise AttributeError(
                '"{}" does not have "name" attribute'.format(cls.__name__),
            )
        elif name is None:
            name = cls.name

        logger.debug('Shared extension "%s"', name)

        return extensions.set(name, cls, to_transport=True, args=args, kwargs=kwargs)

    @staticmethod
    def shared_data(name, data):
        """
        Shared data to Suite and TestCase instances.
        Data will be copied during installation.

        Example::

            data = {
                'a': 1,
                'b': 2,
                'c': 3,
            }
            app = NoseApp('my_app')
            add.shared_data('data', data)

        :param name: extension name
        :type name: str

        :param data: any object
        """
        logger.debug('Shared data "%s"', name)

        return extensions.set(name, data, to_transport=False)

    def register_suite(self, suite, add_prefix=True):
        """
        Add suite to application

        :type suite: noseapp.suite.base.Suite
        """
        logger.debug('Register suite "%s"', suite.name)

        suite.mount_to_app(self)

    def register_suites(self, suites):
        """
        App suite list to application

        :type suites: iter
        """
        for suite in suites:
            self.register_suite(suite)

    def load_suites(self, path, recursive=True, merge_suites=False):
        """
        Auto load suites. Path can be package or simple dir.

        :param path: path to suites dir
        :type path: str

        :param recursive: recursive load from path or load from simple dir
        :type recursive: bool

        :param merge_suites: merge suites from sub application
         to self context. for master application only.
        :type merge_suites: bool
        """
        if path not in sys.path:
            sys.path.append(path)

        if recursive:
            suites = loader.load_suites_from_path(path)
        else:
            suites = loader.load_suites_from_dir(path)

        self.register_suites(suites)

        if merge_suites:
            merge_context(self, merge_suites=True)

    def show_tree(self, show_docs=False, doc_lines=1):
        """
        Show tree of suites.

        :param show_docs: print or not print docs
        :type show_docs: bool

        :param doc_lines: number lines of doc
        :type doc_lines: int
        """
        from noseapp.core.collector.output import tree

        tree(self.suites, show_docs=show_docs, doc_lines=doc_lines, exit=False)

    def run(self):
        """
        To perform run suites.
        If was success after run and not exit param is False then return True.

        Make run from command line::

            noseapp-manage run project.app:create_app

        :rtype: bool
        :raises: RuntimeError
        """
        logger.debug('Run application %s', repr(self))

        if self.__is_sub_app:
            raise RuntimeError(
                'Application {} can not be run. Application is sub application.'.format(self),
            )

        @contextmanager
        def teardown_app():
            try:
                yield
            finally:
                app_callback(self, 'tearDownApp')

        with teardown_app():
            success = self.__test_program.run()

        return success

    def __repr__(self):
        return '<NoseApp({}): {}>'.format(
            self.status, self.__name,
        )



# -*- coding: utf-8 -*-


class SuiteContext(object):
    """
    Context storage of suite instance.

    Usage:

        >>> context = SuiteContext(['extension'])
        >>> context.add_setup(lambda: None)
        >>> ...
    """

    def __init__(self, require):
        """
        :param require: extensions names
        :type require: list
        """
        self.__require = require

        self.__setup = []
        self.__teardown = []

        self.__pre_run = []
        self.__post_run = []

        self.__test_cases = []
        self.__extensions = {}

    @property
    def pre_run(self):
        """
        Pre run callbacks storage.

        :rtype: list
        """
        return self.__pre_run

    @property
    def post_run(self):
        """
        Post run callbacks storage.

        :rtype: list
        """
        return self.__post_run

    @property
    def require(self):
        """
        Require extensions storage.

        :rtype: list
        """
        return self.__require

    @property
    def test_cases(self):
        """
        Test cases storage.

        :rtype: list
        """
        return self.__test_cases

    @property
    def extensions(self):
        """
        Storage for prepared extensions.

        :rtype: dict
        """
        return self.__extensions

    def add_test_case(self, case):
        """
        Add test case class to storage.

        :param case: test case class
        :type case: noseapp.case.base.ToNoseAppTestCase
        """
        self.__test_cases.append(case)

    def add_extension(self, name, ext):
        """
        Add prepared extension to storage.

        :param name: extension name
        :type name: str

        :param ext: prepared extension
        """
        self.__extensions[name] = ext

    def add_pre_run(self, func):
        """
        Add pre run callback to storage.

        :param func: callback for pre run
        :type func: callable
        """
        self.__pre_run.append(func)

    def add_post_run(self, func):
        """
        Add post run callback to storage.

        :param func: callback for post run
        :type func: callable
        """
        self.__post_run.append(func)

    def add_setup(self, func):
        """
        Add setup callback to storage.

        :param func: callback to setup
        :type func: callable
        """
        self.__setup.append(func)

    def add_teardown(self, func):
        """
        Add teardown callback to storage.

        :param func: callback to teardown
        :type func: callable
        """
        self.__teardown.append(func)

    def setup(self):
        """
        Method make call to chain of setup callback storage before run suite.
        For nose.suite.ContextSuite
        """
        for callback in self.__setup:
            callback()

    def teardown(self):
        """
        Method make call to chain of teardown callback storage after run suite.
        For nose.suite.ContextSuite
        """
        for callback in self.__teardown:
            callback()



# -*- coding: utf-8 -*-

import logging

from noseapp.core.constants import RunStrategy


logger = logging.getLogger(__name__)


class ClassFactory(object):
    """
    Factory for creation classes
    """

    def __init__(self, options):
        """
        :param options: nose config options
        """
        if options.run_strategy not in RunStrategy.ALL:
            raise ValueError(
                'Incorrect run strategy: "{}"'.format(options.run_strategy),
            )

        self.__options = options

        self.__current_suite_class = None
        self.__current_runner_class = None

    @property
    def options(self):
        return self.__options

    @property
    def suite_class(self):
        if not self.__current_suite_class:
            return self.create_suite_class()

        return self.__current_suite_class

    @property
    def runner_class(self):
        if not self.__current_runner_class:
            return self.create_runner_class()

        return self.__current_runner_class

    def create_suite_class(self):
        """
        Create class for building suite
        """
        from noseapp.core.suite.base import BaseSuite

        self.__current_suite_class = BaseSuite

        performer_class = BaseSuite.performer_class

        try:
            if self.__options.run_strategy == RunStrategy.GEVENT and self.__options.async_tests:
                from noseapp.core.suite.performers.gevent import GeventSuitePerformer
                performer_class = GeventSuitePerformer

            elif self.__options.run_strategy in (RunStrategy.MULTIPROCESSING, RunStrategy.THREADING)\
                    and self.__options.async_tests:
                from noseapp.core.suite.performers.threading import ThreadSuitePerformer
                performer_class = ThreadSuitePerformer

            else:
                if self.__options.async_tests:
                    from noseapp.core.suite.performers.threading import ThreadSuitePerformer
                    performer_class = ThreadSuitePerformer

        except AttributeError:
            pass

        self.__current_suite_class.performer_class = performer_class

        logger.debug(
            'Current suite performer class is "%s"',
            performer_class.__name__,
        )

        return self.__current_suite_class

    def create_runner_class(self):
        """
        Create test runner class
        """
        from noseapp.core.runner.base import BaseTestRunner

        self.__current_runner_class = BaseTestRunner

        performer_class = BaseTestRunner.performer_class

        try:
            if self.__options.run_strategy == RunStrategy.MULTIPROCESSING\
                    or (self.__options.run_strategy == RunStrategy.SIMPLE and self.__options.async_suites):
                from noseapp.core.runner.performers.multiprocessing import MPRunPerformer
                performer_class = MPRunPerformer

            elif self.__options.run_strategy == RunStrategy.GEVENT:
                from noseapp.core.runner.performers.gevent import GeventRunPerformer
                performer_class = GeventRunPerformer

            elif self.__options.run_strategy == RunStrategy.THREADING:
                from noseapp.core.runner.performers.threading import ThreadingRunPerformer
                performer_class = ThreadingRunPerformer

        except AttributeError:
            pass

        self.__current_runner_class.performer_class = performer_class

        logger.debug('Current run performer class is "%s"', performer_class.__name__)

        return self.__current_runner_class



# -*- coding: utf-8 -*-


# Config can be set from environment:
#
# export NOSEAPP_CONFIG='etc.base'
# export NOSEAPP_CONFIG='/home/user/project/etc/base.py'
CONFIG_CHECKOUT_ENV = 'NOSEAPP_CONFIG'


class RunStrategy(object):
    """
    Strategy to run tests
    """

    SIMPLE = 'simple'
    GEVENT = 'gevent'
    THREADING = 'threading'
    MULTIPROCESSING = 'multiprocessing'

    ALL = (
        SIMPLE,
        GEVENT,
        THREADING,
        MULTIPROCESSING,
    )



# -*- coding: utf-8 -*-

"""
Work with context of application
"""

from noseapp.plugins.base import AppPlugin


def app_callback(master_app, name, *args, **kwargs):
    """
    Wrapper for call callback of application

    :type master_app: noseapp.app.base.NoseApp
    :param name: callback name

    :params args, kwargs: args, kwargs of callback method
    """
    if master_app.is_sub_app:
        raise RuntimeError(
            'Can not call callback "{}" of {}. Application is sub application.'.format(
                name, master_app,
            ),
        )

    apps = [master_app]
    apps.extend(master_app.sub_apps)

    callback = lambda a: getattr(a, name)

    for app in apps:
        callback(app)(*args, **kwargs)


def prepare_plugins(plugins, app):
    """
    Prepare plugins to run

    :param plugins: plugin list
    :tyep app: noseapp.app.base.NoseApp
\    """
    for plugin in plugins:
        if isinstance(plugin, AppPlugin):
            plugin.init_app(app)


def merge_context(
        master_app,
        merge_setup=False,
        merge_suites=False,
        merge_plugins=False,
        merge_teardown=False):
    """
    Merge context from sub apps to master app.

    :param master_app: master app instance
    :type master_app: noseapp.app.base.NoseApp

    :param merge_setup: merge setup callbacks?
    :type merge_setup: bool

    :param merge_suites: merge suites?
    :type merge_suites: bool

    :param merge_plugins: merge plugins?
    :type merge_plugins: bool

    :param merge_teardown: merge teardown callbacks?
    :type merge_teardown: bool

    :raises: RuntimeError
    """
    if master_app.is_sub_app:
        raise RuntimeError(
            'Can not marge context of {}. Application is sub application.'.format(
                master_app,
            ),
        )

    # plugins
    if merge_plugins:
        plugins = []
        plugin_names = map(
            lambda p: p.name,
            master_app.context.plugins,
        )

        for plugin_list in map(lambda sa: sa.plugins, master_app.sub_apps):
            for plugin in plugin_list:
                if plugin.name not in plugin_names:
                    plugins.append(plugin)
                    plugin_names.append(plugin.name)

        master_app.context.plugins.extend(plugins)
        prepare_plugins(master_app.context.plugins, master_app)

    for sub_app in master_app.sub_apps:
        # suites
        if merge_suites:
            master_app.context.suites.extend(
                sub_app.suites,
            )

        # setup callbacks
        if merge_setup:
            master_app.context.setup_callbacks.extend(
                sub_app.context.setup_callbacks,
            )

        # teardown callbacks
        if merge_teardown:
            master_app.context.teardown_callbacks.extend(
                sub_app.context.teardown_callbacks,
            )


class AppContext(object):
    """
    Context storage of NoseApp instance.

    Class is storage for application data.
    Callback functions is here.

    Usage:

        >>> context = AppContext()
        >>> context.add_setup(lambda: print 'Hello World!')
        >>> ...
    """

    def __init__(self):
        # Suites instances
        self.__suites = []
        # Plugins instances
        self.__plugins = []

        # Setup callbacks
        self.__setup = []
        # Teardown callbacks
        self.__teardown = []

    @property
    def suites(self):
        """
        Suites storage.
        After register on application suite will be here.
        """
        return self.__suites

    @property
    def plugins(self):
        """
        Plugins storage.
        After add plugin to application he will be here.
        """
        return self.__plugins

    @property
    def setup_callbacks(self):
        """
        Setup callback storage
        """
        return self.__setup

    @property
    def teardown_callbacks(self):
        """
        Teardown callback storage
        """
        return self.__teardown

    def add_suite(self, suite):
        """
        Add suite to suite storage

        :param suite: suite instance
        :type suite: noseapp.suite.base.Suite
        """
        self.__suites.append(suite)

    def add_setup(self, func):
        """
        Add setup callback to setup callback storage

        :param func: function to call
        :type func: callable
        """
        self.__setup.append(func)

    def add_teardown(self, func):
        """
        Add teardown callback to teardown callback storage

        :param func: function to call
        :type func: callable
        """
        self.__teardown.append(func)

    def setup(self):
        """
        Method make call to chain of setup callback storage before run suites.
        For nose.suite.ContextSuite.
        """
        for callback in self.__setup:
            callback()

    def teardown(self):
        """
        Method make call to chain of teardown callback storage after run suites.
        For nose.suite.ContextSuite.
        """
        for callback in self.__teardown:
            callback()



# -*- coding: utf-8 -*-

from functools import wraps


def flows(*flows):
    """
    Decorator for parametrize test method.

    Usage::

        from noseapp import Suite
        from noseapp import TestCase
        from noseapp.datastructures import Context

        suite = Suite(__name__)

        @suite.register
        class MyTestCase(TestCase):
            @flows(
                Context(integer=1, string='hello'),
                Context(integer=2, string='world'),
            )
            def test(self, ctx):
                ctx.integer
                ctx.string
    """
    def wrapper(f):
        @wraps(f)
        def wrapped(self, *args, **kwargs):
            for flow in flows:
                f(self, flow, *args, **kwargs)
        return wrapped
    return wrapper



# -*- coding: utf-8 -*-

import re
import logging
from random import Random

from noseapp.core import loader
from noseapp.core.collector import output


logger = logging.getLogger(__name__)


CASE_COLLECT_STRATEGY = 'case'
BASIC_COLLECT_STRATEGY = 'basic'
SUITE_COLLECT_STRATEGY = 'suite'
METHOD_COLLECT_STRATEGY = 'method'

COLLECTOR_COMMAND_PATTERN = re.compile(r'^.*\.|^.*:')
COLLECT_CASE_COMMAND_PATTERN = re.compile(r'^.*\:')
COLLECT_METHOD_COMMAND_PATTERN = re.compile(r'^.*\:.*\..*')


def get_command(program_data):
    """
    Get command to collect of program data

    :type program_data: noseapp.core.program.ProgramData
    """
    if program_data.config.options.run_test:
        return program_data.config.options.run_test

    if len(program_data.argv) > 2:
        return program_data.argv[1]

    return None


def get_strategy(command):
    """
    Get strategy by command

    :param command: command to collect
    :type command: str or None
    """
    if command and COLLECTOR_COMMAND_PATTERN.search(command) is not None:

        if COLLECT_METHOD_COMMAND_PATTERN.search(command) is not None:
            return METHOD_COLLECT_STRATEGY

        if COLLECT_CASE_COMMAND_PATTERN.search(command) is not None:
            return CASE_COLLECT_STRATEGY

        return SUITE_COLLECT_STRATEGY

    return BASIC_COLLECT_STRATEGY


class CollectSuite(object):
    """
    Collect suite to run.
    """

    def __init__(self, program_data):
        """
        :type program_data: noseapp.core.program.ProgramData
        """
        if program_data.config.options.ls:
            output.tree(program_data.suites, show_docs=program_data.config.options.doc)

        self.__program_data = program_data
        self.__command = get_command(self.__program_data)
        self.__strategy = get_strategy(self.__command)

    @property
    def command(self):
        """
        Command to collect
        """
        return self.__command

    @property
    def strategy(self):
        """
        Strategy to collect
        """
        return self.__strategy

    @property
    def program_data(self):
        """
        :rtype: noseapp.core.program.ProgramData
        """
        return self.__program_data

    def collect_by_basic_strategy(self):
        """
        Basic collect without rules
        """
        kwargs = {}

        if self.__program_data.config.options.random:
            random = Random(
                self.__program_data.config.options.random_seed,
            )
            random.shuffle(self.__program_data.suites)

            kwargs.update(shuffle=random.shuffle)

        return [
            suite(
                self.__program_data,
                **kwargs
            )
            for suite in self.__program_data.suites
        ]

    def collect_by_suite_strategy(self):
        """
        Collect suite from command
        """
        suite = loader.load_suite_by_name(
            self.__command,
            self.__program_data.suites,
        )

        return [suite(self.__program_data)]

    def collect_by_case_strategy(self):
        """
        Collect case of suite from command
        """
        suite_name, case_name = self.__command.split(':')
        suite = loader.load_suite_by_name(
            suite_name,
            self.__program_data.suites,
        )

        return [
            suite(
                self.__program_data,
                case_name=case_name,
            ),
        ]

    def collect_by_method_strategy(self):
        """
        Collect case method of suite from command
        """
        suite_name, case_info = self.__command.split(':')
        case_name, method_name = case_info.split('.')
        suite = loader.load_suite_by_name(
            suite_name,
            self.__program_data.suites,
        )

        return [
            suite(
                self.__program_data,
                case_name=case_name,
                method_name=method_name,
            ),
        ]

    @property
    def collect(self):
        """
        Main collect
        """
        strategy_to_method = {
            CASE_COLLECT_STRATEGY: self.collect_by_case_strategy,
            BASIC_COLLECT_STRATEGY: self.collect_by_basic_strategy,
            SUITE_COLLECT_STRATEGY: self.collect_by_suite_strategy,
            METHOD_COLLECT_STRATEGY: self.collect_by_method_strategy,
        }

        logger.debug('Strategy for collect suites is "%s"', self.__strategy)

        return strategy_to_method[self.__strategy]


def collect(program_data, collector_class=CollectSuite):
    """
    Function is wrapper for calling to collector class
    """
    collector = collector_class(program_data)

    return program_data.suite_class(
        collector.collect(),
        config=program_data.config,
        context=program_data.context,
    )



# -*- coding: utf-8 -*-

from nose.case import Test as NoseTestWrapper
from unittest import TestCase as BaseTestCase

from noseapp.case.context import TestCaseContext
from noseapp.datastructures import ModifyDict as MountData


def make_test_case_class_from_function(
        func,
        base_class,
        doc=None,
        simple=False,
        class_name=None,
        class_name_creator=None):
    """
    It's just create test case class from function.
    Function will be bound method (runTest) of new class.

    :param func: test function. for example: my_test = lambda case: 'just do it something'
    :param base_class: base class for making class
    :param simple: if true that not inject case instance
    :param class_name: name of new class. default function name
    :param class_name_creator: callable object for creating name of function object
    """
    if callable(class_name_creator):
        class_name = class_name_creator(func)

    cls = type(
        class_name or func.__name__,
        (base_class, ),
        {
            '__doc__': doc or func.__doc__,
        },
    )

    if simple:
        cls.runTest = lambda s: func()
    else:
        cls.runTest = func

    return cls


def case_is_mount(case):
    """
    If case is mounting to suite then True else False

    :param case: test case class or instance

    :rtype: bool
    """
    return hasattr(case, '__mount_data__') \
        and \
        isinstance(case.__mount_data__, MountData)


def get_case_master_id(case):
    """
    Get id of master process

    :type case: ToNoseAppTestCase
    """
    if isinstance(case, NoseTestWrapper):
        case = case.test

    return case._ToNoseAppTestCase__master_id


class ToNoseAppTestCase(object):
    """
    This is mixin for noseapp supporting.
    Class must be first in inheritance chain!

    Usage::

        class MyTestCase(ToNoseAppTestCase, unittest.TestCase):
            pass
    """

    def __init__(self, *args, **kwargs):
        if not case_is_mount(self):
            raise RuntimeError(
                'Test case "{}" can not be created without mounting to suite'.format(
                    self.__class__,
                ),
            )

        super(ToNoseAppTestCase, self).__init__(*args, **kwargs)

        self.__master_id = id(self)

        if hasattr(self, 'REQUIRE'):
            self.__mount_data__.context.update_by_require(self.REQUIRE)

    @classmethod
    def mount_to_suite(cls, suite):
        """
        Mount class to suite.
        If this procedure will be ignored then class can't be instantiate.

        :type suite: noseapp.suite.base.Suite

        :raises: RuntimeError
        :rtype: cls
        """
        if case_is_mount(cls):
            raise RuntimeError(
                '"{}" already mount to {}'.format(
                    cls.__name__, cls.__mount_data__.of_suite,
                ),
            )

        setattr(
            cls,
            '__mount_data__',
            MountData(
                of_suite=suite.name,
                context=TestCaseContext(suite),
            ),
        )

        return cls

    @property
    def of_suite(self):
        """
        This is suite name after mounting.
        """
        return self.__mount_data__.of_suite

    def ext(self, name):
        """
        Get extension by name.
        Extension must be required.

        Example::

            suite = Suite(__name__, require=['extension'])

            @suite.register
            class Test(TestCase):

                def test(self):
                    ext = self.ext('extension')

        :param name: extension name
        :type name: str

        :raises: noseapp.core.extensions.ExtensionNotRequired
        """
        return self.__mount_data__.context.ext(name)

    def __str__(self):
        return '{} ({}:{})'.format(
            self._testMethodName,
            self.__mount_data__.of_suite,
            self.__class__.__name__,
        )


class TestCase(ToNoseAppTestCase, BaseTestCase):
    """
    Base case class
    """
    pass



# -*- coding: utf8 -*-

import sys

from nose.suite import ContextSuite


def get_suite_master_id(suite):
    """
    Get id of master process
    """
    return suite._BaseSuite__master_id


class SuitePerformer(object):
    """
    Perform run suite
    """

    def __init__(self, suite, result, orig):
        self.orig = orig
        self.suite = suite
        self.result = result

    def run_one_test(self, test):
        """
        One test of suite to run
        """
        for callback in self.suite.pre_run_handlers:
            try:
                callback(test.test)
            except KeyboardInterrupt:
                raise
            except:
                self.orig.addError(test, sys.exc_info())
                return

        test(self.orig)

        for callback in self.suite.post_run_handlers:
            try:
                callback(test.test)
            except KeyboardInterrupt:
                raise
            except:
                self.orig.addError(test, sys.exc_info())

    def __call__(self, **kwargs):
        for test in self.suite.tests:
            if self.result.shouldStop:
                break

            if isinstance(test, BaseSuite):
                test.run(self.result, **kwargs)
                continue

            self.run_one_test(test)


class BaseSuite(ContextSuite):
    """
    Base suite
    """

    performer_class = SuitePerformer

    def __init__(self, *args, **kwargs):
        self.__master_id = id(self)
        self.__pre_run_handlers = kwargs.pop('pre_run_handlers', [])
        self.__post_run_handlers = kwargs.pop('post_run_handlers', [])

        super(BaseSuite, self).__init__(*args, **kwargs)

    @property
    def tests(self):
        return self._tests

    @property
    def pre_run_handlers(self):
        return self.__pre_run_handlers

    @property
    def post_run_handlers(self):
        return self.__post_run_handlers

    def run(self, result, **kwargs):
        if self.resultProxy:
            result, orig = self.resultProxy(result, self), result
        else:
            result, orig = result, result

        try:
            self.setUp()
        except KeyboardInterrupt:
            raise
        except:
            self.error_context = 'setup'
            result.addError(self, self._exc_info())
            return

        try:
            self.performer_class(self, result, orig)(**kwargs)
        finally:
            self.has_run = True

            try:
                self.tearDown()
            except KeyboardInterrupt:
                raise
            except:
                self.error_context = 'teardown'
                result.addError(self, self._exc_info())



# -*- coding: utf8 -*-

"""
Core of framework.

If you write modules of core and they're using other modules
of library that make using import inside functions or methods.
"""

from noseapp.core.program import TestProgram


class ExtensionInstaller(object):
    """
    Container for extension.
    Lazy creating instance.
    """

    def __init__(self, cls, args, kwargs):
        self.name = cls.name

        self.cls = cls
        self.args = args
        self.kwargs = kwargs

    def __call__(self):
        return self.cls(*self.args, **self.kwargs)


__all__ = (
    TestProgram,
    ExtensionInstaller,
)



# -*- coding: utf-8 -*-

"""
A powerful, extensible and easy to use framework for test development

By Trifonov Mikhail

For support, use the https://github.com/noseapp/noseapp/issues tracker
"""

from noseapp.app.base import NoseApp

from noseapp.suite.base import Suite

from noseapp.case import step
from noseapp.case import TestCase
from noseapp.case import ScreenPlayCase

from noseapp.plugins import AppPlugin


__all__ = (
    step,
    Suite,
    NoseApp,
    TestCase,
    AppPlugin,
    ScreenPlayCase,
)



# -*- coding: utf-8 -*-

from noseapp.core import extensions


class TestCaseContext(object):
    """
    Context of test case instance
    """

    def __init__(self, suite):
        """
        :param suite: suite instance
        :type suite: noseapp.suite.base.Suite
        """
        self.__extensions = dict(
            (k, v) for k, v in suite.context.extensions.items()
        )
        self.__require = [r for r in suite.require]

    @property
    def require(self):
        """
        Extensions names storage

        :rtype: list
        """
        return self.__require

    def update_by_require(self, require):
        """
        Update extension storage by require from extensions tmp

        :param require: extensions names
        :type require: list
        """
        self.__require.extend(require)

        for ext_name in require:
            self.__extensions[ext_name] = extensions.get(ext_name)

    def ext(self, name):
        """
        Get extension by name

        :param name: extension name

        :raises: core.extensions.ExtensionNotRequired
        """
        if name in self.__require:
            return self.__extensions[name]

        raise extensions.ExtensionNotRequired(name)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from contextlib import contextmanager
from multiprocessing import cpu_count

from noseapp.utils.common import waiting_for
from noseapp.core.suite.base import BaseSuite
from noseapp.case.base import get_case_master_id
from noseapp.utils.common import TimeoutException
from noseapp.core.runner.base import RunPerformer
from noseapp.core.suite.base import get_suite_master_id


DEFAULT_RELEASE_TIMEOUT = 180  # default timeout for release processes list
DEFAULT_MAX_PROCESSES = cpu_count()  # default max processes num to run in a moment


@contextmanager
def terminate_processes(runner):
    """
    Do something with runner and
    terminate processes after this

    :type runner: MPSuiteRunner
    """
    try:
        yield
    finally:
        runner.terminate()
        runner.join()


def get_master_id(obj):
    """
    Get id of master process
    """
    if isinstance(obj, BaseSuite):
        return get_suite_master_id(obj)
    return get_case_master_id(obj)


def target(suite, mp_result):
    """
    Task to perform
    """
    suite(mp_result)
    mp_result.save_result()


class MPResult(object):
    """
    Sync result between processes
    """

    MATCH = {}  # id -> instance

    def __init__(self, result):
        from multiprocessing import Manager

        # Test result instance
        self.result = result

        # Result queue
        self.queue = Manager().Queue()

    def __getattr__(self, item):
        return getattr(self.result, item)

    @staticmethod
    def pack_result_storage(storage):
        """
        Pack result from storage
        """
        return [(get_master_id(s[0]), s[1]) for s in storage]

    def unpack_result_storage(self, storage):
        """
        Unpack result from storage
        """
        unpack_storage = []

        for master_id, message in storage:
            unpack_storage.append(
                (self.MATCH[master_id], message),
            )

        return unpack_storage

    def match(self, suite):
        """
        Match id of master process to instance
        """
        self.MATCH[get_suite_master_id(suite)] = suite

        def match(s):
            for o in s:
                if isinstance(o, BaseSuite):
                    self.MATCH[get_suite_master_id(o)] = o
                    match(o)
                else:
                    self.MATCH[get_case_master_id(o)] = o

        match(suite)

    def save_result(self):
        """
        Save result in queue
        """
        self.queue.put(
            (
                (
                    self.pack_result_storage(self.result.errors),
                    self.pack_result_storage(self.result.skipped),
                    self.pack_result_storage(self.result.failures),
                ),
                self.result.testsRun,
            ),
        )

    def make_result(self):
        """
        Merge result from queue to result instance
        """
        while not self.queue.empty():
            (errors, skipped, failures), run_tests = self.queue.get()

            self.result.errors.extend(self.unpack_result_storage(errors))
            self.result.skipped.extend(self.unpack_result_storage(skipped))
            self.result.failures.extend(self.unpack_result_storage(failures))

            self.result.testsRun += run_tests


class MasterProcess(object):
    """
    Run suites with multiprocessing
    """

    def __init__(self,
                 result,
                 max_processes=DEFAULT_MAX_PROCESSES,
                 release_timeout=DEFAULT_RELEASE_TIMEOUT):
        from multiprocessing import Process

        # Timeout for release stack
        self.release_timeout = release_timeout
        # Max processes to run
        self.max_processes = max_processes if max_processes > 0 else DEFAULT_MAX_PROCESSES

        # Tasks queue
        self.queue = []
        # Run processes stack
        self.stack = []

        # Class for create process
        self.process_class = Process

        # Result wrapper for sync
        self.mp_result = MPResult(result)

    def add_suite(self, suite):
        """
        Add suite to run
        """
        self.mp_result.match(suite)

        self.queue.append(
            self.process_class(
                target=target,
                args=(suite, self.mp_result),
            ),
        )

    def add_suites(self, suites):
        """
        Add suites to run. For usability only.
        """
        for suite in suites:
            self.add_suite(suite)

    def try_release(self):
        """
        Find not alive processes in stack and remove their
        """
        for process in self.stack:
            if not process.is_alive():
                process.terminate()
                process.join()
                self.stack.remove(process)

    def is_release(self):
        """
        Will be True if free place in stack else False
        """
        self.try_release()
        return len(self.stack) < self.max_processes

    def wait_release(self):
        """
        To wait free place in stack
        """
        try:
            waiting_for(self.is_release, timeout=self.release_timeout, sleep=0.001)
        except TimeoutException:
            raise TimeoutException(
                'Process list have not been release for "{}" sec.'.format(self.release_timeout),
            )

    def terminate(self):
        """
        Terminate processes from stack
        """
        for process in self.stack:
            process.terminate()

    def join(self):
        """
        Join processes from stack
        """
        for process in self.stack:
            process.join(timeout=self.release_timeout)

    def run(self):
        """
        Run suites
        """
        while self.queue:
            process = self.queue.pop()
            process.start()
            self.stack.append(process)
            self.wait_release()

        self.join()
        self.mp_result.make_result()


class MPRunPerformer(RunPerformer):

    def __call__(self, suites, result):
        max_size = self.runner.config.options.async_suites
        timeout = self.runner.config.options.multiprocessing_timeout

        # XXX:
        # We must prepare test result here for correct finalize.
        # Plugins chain will be still called with run suite
        # on resultProxy, but master process don't know about this
        self.runner.config.plugins.prepareTestResult(result)

        process = MasterProcess(
            result,
            max_processes=max_size,
            release_timeout=timeout,
        )
        process.add_suites(suites)

        with terminate_processes(process):
                process.run()
